/* udp_test.cpp
 *
 * Send out a Mercury UDP SSDP request to find the NFS and HTTP servers.
 *
 * (C) 2002-2003 Roger Lipscombe, http://www.differentpla.net/
 */

#if defined(WIN32)
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <winsock2.h>
#define fd_set FD_SET
typedef int socklen_t;
#else
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
typedef int SOCKET;
#define INVALID_SOCKET (-1)
typedef struct sockaddr SOCKADDR;
typedef struct sockaddr_in SOCKADDR_IN;

#define closesocket(fd) close((fd))
#endif

#include <stdio.h>
#include <string.h>

#define MERCURY_HTTP_UPNP_GUID "1D274DB0-F053-11d3-BF72-0050DA689B2F"
#define MERCURY_HTTP_UPNP_STRING "upnp:uuid:" MERCURY_HTTP_UPNP_GUID
#define MERCURY_NFS_UPNP_GUID "1D274DB1-F053-11d3-BF72-0050DA689B2F"
#define MERCURY_NFS_UPNP_STRING "upnp:uuid:" MERCURY_NFS_UPNP_GUID

void find_servers(const char *server_description, const char *upnp_string);

int main()
{
#if defined(WIN32)
	WSADATA wsa;
	WSAStartup(MAKEWORD(2,0), &wsa);
#endif

	find_servers("NFS", MERCURY_NFS_UPNP_STRING);
	find_servers("HTTP", MERCURY_HTTP_UPNP_STRING);

#if defined(WIN32)
	WSACleanup();
#endif

	return 0;
}

void find_servers(const char *server_description, const char *upnp_string)
{
	printf("\nLooking for Rio Receiver %s Servers...\n", server_description);

	// We send a request formatted as:
	// MERCURY_UPNP_STRING\n
	// MAC_ADDRESS\n

	char buffer[128];
	int buffer_length = sprintf(buffer, "%s\n%s\n", upnp_string, "00:00:00:00:00:00");

	// We send it to the all-ones address.
	SOCKET soc = socket(PF_INET, SOCK_DGRAM, 0);
	if (soc != INVALID_SOCKET)
	{
#if 0
		SOCKADDR_IN from;
		memset(&from, 0, sizeof(SOCKADDR_IN));
		from.sin_family = AF_INET;
		from.sin_port = htons(21075);
		from.sin_addr.s_addr = htonl(INADDR_ANY);
		int bind_result = bind(soc, (SOCKADDR *)&from, sizeof(SOCKADDR_IN));
		if (bind_result != 0)
		{
			fprintf(stderr, "Warning: bind failed: %d\n", WSAGetLastError());
		}
#endif
		int allow_broadcast = 1;
		setsockopt(soc, SOL_SOCKET, SO_BROADCAST, (const char *)&allow_broadcast, sizeof(allow_broadcast));

		SOCKADDR_IN to;
		memset(&to, 0, sizeof(SOCKADDR_IN));
		to.sin_family = AF_INET;
		to.sin_port = htons(21075);

		to.sin_addr.s_addr = htonl(INADDR_BROADCAST);
		//	to.sin_addr.s_addr = htonl(0x0a2301ff);

		printf("Sending request to %s..", inet_ntoa(to.sin_addr));
		bool found_server = false;
		for (int retries = 0; !found_server && retries < 5; ++retries)
		{
			sendto(soc, buffer, buffer_length, 0, (SOCKADDR*)&to, sizeof(SOCKADDR_IN));

			// Now wait for some responses
			fd_set readfds;
			FD_ZERO(&readfds);
			FD_SET(soc, &readfds);

			struct timeval tv;
			tv.tv_sec = 2<<retries;
			tv.tv_usec = 0;
			printf(".");
			int select_result = select(1, &readfds, NULL, NULL, &tv);
			if (select_result == 0)
			{
				// Timeout
			}
			else if (select_result < 0)
			{
				// Error
			}
			else
			{
				// Activity -- select on the socket with short timeouts until we've read all of the responses
				for (;;)
				{
					struct timeval tv;
					tv.tv_sec = 1;
					tv.tv_usec = 0;
					int select_result = select(1, &readfds, NULL, NULL, &tv);
					if (select_result == 0)
					{
						break;
					}
					else if (select_result < 0)
					{
						// Error
					}
					else
					{
						// We've actually got some data, show the user.
						SOCKADDR_IN from;
						socklen_t fromlen = sizeof(SOCKADDR_IN);
						char buffer2[128];
						int buffer2_length = sizeof(buffer2);

						printf("\n");
						int bytes_read = recvfrom(soc, buffer2, buffer2_length, 0, (SOCKADDR*)&from, &fromlen);
						if (bytes_read > 0)
						{
							buffer2[bytes_read] = 0;
							printf("%s replied: %s\n", inet_ntoa(from.sin_addr), buffer2);

							found_server = true;
						}
					}
				}
			}
		}

		closesocket(soc);
	}
	else
	{
		fprintf(stderr, "Couldn't create datagram socket\n");
	}
}
